#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Crée mars 2024
@auteur: David ALBERTO (www.astrolabe-science.fr)

Ce code utilise le module mendeleev (données sur les éléments chimiques)
Dans ce module, un élément est appelé par element(N).
Le script trace la composition isotopique d'un élément chimique,
sous forme d'histogramme.
Les noyaux magiques et les noyaux doublement magiques sont colorés distinctement.
Basé sur le module mendeleev (https://mendeleev.readthedocs.io/)
"""
 # Paramètres personnalisables-----
couleur_histo = 'skyblue'
couleur_magique = 'lightcoral'
couleur_doublemagique = 'blueviolet'
largeur = 0.2  # largeur des rectangles
# ---------------------------------
liste_magique = [2, 8, 20, 28, 50, 82, 126]
# Appel des modules
import matplotlib.pyplot as plt
from mendeleev.fetch import fetch_table

isotopes = fetch_table('isotopes', index_col='id') # récupération des données
# ---------------------------------
ptable  =  fetch_table('elements')
isotopes  =  fetch_table('isotopes', index_col = 'id')
colonnes = ['atomic_number','mass_number','abundance','is_radioactive','mass']
donnees = isotopes[colonnes]
donnees = donnees[donnees.is_radioactive == False] # supprime les instables

def data(Z):
    """
    Parameters
    ----------
    Z : TYPE : (entier) : numéro atomique
    Returns
    -------
    liste x : nombres de masse des isotopes de l'élément.
    liste y = abondance des isotopes.
    """
    data = donnees[donnees.atomic_number == Z]
    data = data.dropna()  # supprime les données absentes
    liste_A = data.mass_number
    liste_abo = data.abundance
    return liste_A, liste_abo

def est_magique(Z,N):
    """
    Parameters
    ----------
    Z : TYPE entier
        DESCRIPTION = numéro atomique
    A : TYPE : entier
        DESCRIPTION nombre de neutrons
    Returns
    -------
    booléen : True si le noyau est magique.
    """
    if Z in liste_magique:
        magique = True
    elif N in liste_magique:
        magique = True
    else:magique = False
    return magique

def double_magique(Z,N):
    """
    Parameters
    ----------
    Z : TYPE entier
        DESCRIPTION = numéro atomique
    A : TYPE : entier
        DESCRIPTION nombre de neutrons
    Returns
    -------
    booléen : True si le noyau est doublement magique.
    """
    if Z in liste_magique and N in liste_magique:
        magique = True
    else:
        magique = False
    return magique

def liste_couleurs(Z, liste_A):
    """
    Parameters
    ----------
    Z : TYPE entier DESCRIPTION : numéro atomique de l'élément.
    liste_A : TYPE : liste de strings
        DESCRIPTION. : liste de nombres de masse des isotopes
    Returns
    -------
    liste_c : TYPE : liste de strings
        DESCRIPTION. : couleurs à affecter aux isotopes de l'élément.
    """
    liste_c = []
    for i in range(len(liste_A)):
        N = liste_A[i] - Z # nombre de neutrons
        if double_magique(Z, N):
            coul = couleur_doublemagique
        elif est_magique(Z, N):
            coul = couleur_magique
        else:
            coul = couleur_histo
        liste_c.append(coul)
    return liste_c

def histogramme(Z):
    """
    Parameters
    ----------
    Z : TYPE : entier
        DESCRIPTION : numéro atomique
    Returns
    -------
    Trace l'histogramme des isotopes stables et leur abondance.
    """
    # paramètres du graphique
    fig = plt.figure(tight_layout=True, facecolor='lavender')
    ax = plt.subplot(111)
    ax.set_xlabel('nombre de masse A')
    ax.set_yticks([])
    x, y = data(Z)
    if len(y) >0:
        if len(x)==1:
            ax.set_xlim(x.iloc[0]-3, x.iloc[0]+3)
        else:
            ax.set_ylim(0, max(y)+6) # laisse la place pour le %
        ax.set_xticks(range(min(x), max(x)+1)) # graduations entières en x
        couleur = liste_couleurs(Z, list(x)) # liste de couleurs pour les rectangles
        ax.bar(x, y, width = largeur, ec='k', color = couleur) # tracé de l'histogramme
        nom = noms[Z-1] # nom de l'élément en français
        ax.set_title(f"{Z}-{nom}", fontsize=12, fontweight='bold')
        labels = [f"{y.iloc[i]}%" for i in range(len(y))] # les %
        """
        Création d'une liste de couleurs pour les pourcentages (colorés si 
        l'isotope est magique, noir sinon), en remplaçant la couleur bleue
        dans la liste 'couleur' par noir.
        """
        couleurs_labels = list(map(lambda x: x.replace(couleur_histo, 'k'), couleur))
        # affichage des % :
        for i in range(len(x)):
            ax.text(x.iloc[i], y.iloc[i], labels[i], ha = 'center', va = 'bottom',
                    color = couleurs_labels[i]
                    )
        ax.text(1.005,0.02,"www.astrolabe-science.fr", c= 'grey',
                rotation=90, fontsize=6, transform=ax.transAxes)
        NomFichier = "isotopes_histo" + str(Z)
        fig.savefig(NomFichier+'.png',dpi=200)
        fig.savefig(NomFichier+'.pdf')

#---------------------------------------------------------
noms = ['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennesse','Oganesson',
]#le module mendeleev n'a que les noms des éléments en anglais
# ---------------------------------

# programme principal :
plt.rcParams["font.family"] = "Roboto Condensed"  # ou autre police installée

for Z in range(1,83):
    histogramme(Z)
